diff --git a/NvTriStrip/NvTriStripObjects.cpp b/NvTriStrip/NvTriStripObjects.cpp
index 738169d3f63786a540e576f6a6839e6c9abf37ec..d2c3247866136ad12afe48606656896b72b3b3db 100644
--- a/NvTriStrip/NvTriStripObjects.cpp
+++ b/NvTriStrip/NvTriStripObjects.cpp
@@ -148,7 +148,7 @@ void NvStripifier::BuildStripifyInfo(NvFaceInfoVec &faceInfos, NvEdgeInfoVec &ed
 		{
 			if (edgeInfo01->m_face1 != NULL)
 			{
-				printf("BuildStripifyInfo: > 2 triangles on an edge... uncertain consequences\n");
+				//printf("BuildStripifyInfo: > 2 triangles on an edge... uncertain consequences\n");
 			}
 			else
 			{
@@ -179,7 +179,7 @@ void NvStripifier::BuildStripifyInfo(NvFaceInfoVec &faceInfos, NvEdgeInfoVec &ed
 		{
 			if (edgeInfo12->m_face1 != NULL)
 			{
-				printf("BuildStripifyInfo: > 2 triangles on an edge... uncertain consequences\n");
+				//printf("BuildStripifyInfo: > 2 triangles on an edge... uncertain consequences\n");
 			}
 			else
 			{
@@ -210,7 +210,7 @@ void NvStripifier::BuildStripifyInfo(NvFaceInfoVec &faceInfos, NvEdgeInfoVec &ed
 		{
 			if (edgeInfo20->m_face1 != NULL)
 			{
-				printf("BuildStripifyInfo: > 2 triangles on an edge... uncertain consequences\n");
+				//printf("BuildStripifyInfo: > 2 triangles on an edge... uncertain consequences\n");
 			}
 			else
 			{
@@ -449,22 +449,22 @@ inline int NvStripifier::GetNextIndex(const WordVec &indices, NvFaceInfo *face){
 	
 	if (fv0 != v0 && fv0 != v1){
 		if ((fv1 != v0 && fv1 != v1) || (fv2 != v0 && fv2 != v1)){
-			printf("GetNextIndex: Triangle doesn't have all of its vertices\n");
-			printf("GetNextIndex: Duplicate triangle probably got us derailed\n");
+			//printf("GetNextIndex: Triangle doesn't have all of its vertices\n");
+			//printf("GetNextIndex: Duplicate triangle probably got us derailed\n");
 		}
 		return fv0;
 	}
 	if (fv1 != v0 && fv1 != v1){
 		if ((fv0 != v0 && fv0 != v1) || (fv2 != v0 && fv2 != v1)){
-			printf("GetNextIndex: Triangle doesn't have all of its vertices\n");
-			printf("GetNextIndex: Duplicate triangle probably got us derailed\n");
+			//printf("GetNextIndex: Triangle doesn't have all of its vertices\n");
+			//printf("GetNextIndex: Duplicate triangle probably got us derailed\n");
 		}
 		return fv1;
 	}
 	if (fv2 != v0 && fv2 != v1){
 		if ((fv0 != v0 && fv0 != v1) || (fv1 != v0 && fv1 != v1)){
-			printf("GetNextIndex: Triangle doesn't have all of its vertices\n");
-			printf("GetNextIndex: Duplicate triangle probably got us derailed\n");
+			//printf("GetNextIndex: Triangle doesn't have all of its vertices\n");
+			//printf("GetNextIndex: Duplicate triangle probably got us derailed\n");
 		}
 		return fv2;
 	}
diff --git a/include/gen/TexDesc.h b/include/gen/TexDesc.h
index 61ae63400bbd71176971b3e2e75688a5ecc94794..5373ed34e916f46b090e21d18a66e12e005125a7 100644
--- a/include/gen/TexDesc.h
+++ b/include/gen/TexDesc.h
@@ -34,8 +34,8 @@ struct NIFLIB_API TexDesc {
 	 */
 	TexFilterMode filterMode;
 	/*!
-	 * The texture coordinate set in NiTriBasedGeomData that this texture
-	 * slot will use.
+	 * The texture coordinate set in NiGeometryData that this texture slot
+	 * will use.
 	 */
 	uint uvSet;
 	/*!
diff --git a/include/gen/obj_defines.h b/include/gen/obj_defines.h
index b466d94a1cbbc47ff0dc67080b93c1c8a373cd83..ad863b2421c58a21bad02937b1f4822ab235499a 100644
--- a/include/gen/obj_defines.h
+++ b/include/gen/obj_defines.h
@@ -153,14 +153,16 @@ Ref<NiInterpolator > interpolator; \
 #define A_P_SYS_CTLR_MEMBERS \
 string modifierName; \
 
-#define NI_TRI_BASED_GEOM_MEMBERS \
-Ref<NiTriBasedGeomData > data; \
+#define NI_GEOMETRY_MEMBERS \
+Ref<NiGeometryData > data; \
 Ref<NiSkinInstance > skinInstance; \
 bool hasShader; \
 string shaderName; \
 Ref<NiObject > unknownLink; \
 
-#define NI_TRI_BASED_GEOM_DATA_MEMBERS \
+#define NI_TRI_BASED_GEOM_MEMBERS \
+
+#define NI_GEOMETRY_DATA_MEMBERS \
 string name; \
 mutable ushort numVertices; \
 ushort unknownShort1; \
@@ -182,6 +184,8 @@ vector< vector<TexCoord > > uvSets; \
 ushort unknownShort2; \
 Ref<NiObject > unknownLink; \
 
+#define NI_TRI_BASED_GEOM_DATA_MEMBERS \
+
 #define A_P_SYS_DATA_MEMBERS \
 bool hasUnknownFloats1; \
 vector<float > unknownFloats1; \
@@ -1242,7 +1246,9 @@ CompareMode function; \
 #define A_BONE_L_O_D_CONTROLLER_MEMBERS
 #define NI_SINGLE_INTERPOLATOR_CONTROLLER_MEMBERS
 #define A_P_SYS_CTLR_MEMBERS
+#define NI_GEOMETRY_MEMBERS
 #define NI_TRI_BASED_GEOM_MEMBERS
+#define NI_GEOMETRY_DATA_MEMBERS
 #define NI_TRI_BASED_GEOM_DATA_MEMBERS
 #define A_P_SYS_DATA_MEMBERS
 #define BHK_BLEND_COLLISION_OBJECT_MEMBERS
@@ -1597,21 +1603,31 @@ CompareMode function; \
 #define A_P_SYS_CTLR_PARENT NiSingleInterpolatorController
 
 #define A_P_SYS_CTLR_CONSTRUCT 
-#define NI_TRI_BASED_GEOM_INCLUDE "NiAVObject.h"
+#define NI_GEOMETRY_INCLUDE "NiAVObject.h"
+
+#define NI_GEOMETRY_PARENT NiAVObject
+
+#define NI_GEOMETRY_CONSTRUCT  : data(NULL), skinInstance(NULL), hasShader(false), unknownLink(NULL)
+
+#define NI_TRI_BASED_GEOM_INCLUDE "NiGeometry.h"
+
+#define NI_TRI_BASED_GEOM_PARENT NiGeometry
 
-#define NI_TRI_BASED_GEOM_PARENT NiAVObject
+#define NI_TRI_BASED_GEOM_CONSTRUCT 
+#define NI_GEOMETRY_DATA_INCLUDE "NiObject.h"
 
-#define NI_TRI_BASED_GEOM_CONSTRUCT  : data(NULL), skinInstance(NULL), hasShader(false), unknownLink(NULL)
+#define NI_GEOMETRY_DATA_PARENT NiObject
 
-#define NI_TRI_BASED_GEOM_DATA_INCLUDE "NiObject.h"
+#define NI_GEOMETRY_DATA_CONSTRUCT  : numVertices((ushort)0), unknownShort1((ushort)0), hasVertices(false), numUvSets2((byte)0), unknownByte((byte)0), hasNormals(false), radius(0.0f), hasVertexColors(false), numUvSets((ushort)0), hasUv(false), unknownShort2((ushort)0), unknownLink(NULL)
 
-#define NI_TRI_BASED_GEOM_DATA_PARENT NiObject
+#define NI_TRI_BASED_GEOM_DATA_INCLUDE "NiGeometryData.h"
 
-#define NI_TRI_BASED_GEOM_DATA_CONSTRUCT  : numVertices((ushort)0), unknownShort1((ushort)0), hasVertices(false), numUvSets2((byte)0), unknownByte((byte)0), hasNormals(false), radius(0.0f), hasVertexColors(false), numUvSets((ushort)0), hasUv(false), unknownShort2((ushort)0), unknownLink(NULL)
+#define NI_TRI_BASED_GEOM_DATA_PARENT NiGeometryData
 
-#define A_P_SYS_DATA_INCLUDE "NiTriBasedGeomData.h"
+#define NI_TRI_BASED_GEOM_DATA_CONSTRUCT 
+#define A_P_SYS_DATA_INCLUDE "NiGeometryData.h"
 
-#define A_P_SYS_DATA_PARENT NiTriBasedGeomData
+#define A_P_SYS_DATA_PARENT NiGeometryData
 
 #define A_P_SYS_DATA_CONSTRUCT  : hasUnknownFloats1(false), unknownShort3((ushort)0), hasUnknownFloats2(false), unknownByte1((byte)0)
 
@@ -1802,9 +1818,9 @@ CompareMode function; \
 #define NI_AMBIENT_LIGHT_PARENT NiLight
 
 #define NI_AMBIENT_LIGHT_CONSTRUCT 
-#define NI_AUTO_NORMAL_PARTICLES_DATA_INCLUDE "NiTriBasedGeomData.h"
+#define NI_AUTO_NORMAL_PARTICLES_DATA_INCLUDE "NiGeometryData.h"
 
-#define NI_AUTO_NORMAL_PARTICLES_DATA_PARENT NiTriBasedGeomData
+#define NI_AUTO_NORMAL_PARTICLES_DATA_PARENT NiGeometryData
 
 #define NI_AUTO_NORMAL_PARTICLES_DATA_CONSTRUCT  : numParticles((ushort)0), size(0.0f), numActive((ushort)0), unknownShort((ushort)0), hasSizes(false)
 
@@ -2170,9 +2186,9 @@ CompareMode function; \
 
 #define NI_PARTICLE_ROTATION_CONSTRUCT  : unknownByte((byte)0), unknownFloat1(0.0f), unknownFloat2(0.0f), unknownFloat3(0.0f), unknownFloat4(0.0f)
 
-#define NI_PARTICLES_INCLUDE "NiTriBasedGeom.h"
+#define NI_PARTICLES_INCLUDE "NiGeometry.h"
 
-#define NI_PARTICLES_PARENT NiTriBasedGeom
+#define NI_PARTICLES_PARENT NiGeometry
 
 #define NI_PARTICLES_CONSTRUCT 
 #define NI_AUTO_NORMAL_PARTICLES_INCLUDE "NiParticles.h"
diff --git a/include/obj/NiGeometry.h b/include/obj/NiGeometry.h
new file mode 100644
index 0000000000000000000000000000000000000000..65304f295b0ac4bf718862f6f1344e376d0963b9
--- /dev/null
+++ b/include/obj/NiGeometry.h
@@ -0,0 +1,86 @@
+/* Copyright (c) 2006, NIF File Format Library and Tools
+All rights reserved.  Please see niflib.h for licence. */
+
+#ifndef _NIGEOMETRY_H_
+#define _NIGEOMETRY_H_
+
+#include "NiAVObject.h"
+
+// Include structures
+#include "../Ref.h"
+#include "../gen/SkinWeight.h"
+
+namespace Niflib {
+
+// Forward define of referenced blocks
+class NiObject;
+class NiGeometryData;
+class NiSkinInstance;
+class NiNode;
+
+#include "../gen/obj_defines.h"
+
+class NiGeometry;
+typedef Ref<NiGeometry> NiGeometryRef;
+
+/*!
+ * NiGeometry - Describes a visible scene element with vertices like a
+ * mesh, a particle system, lines, etc.
+ */
+
+class NIFLIB_API NiGeometry : public NI_GEOMETRY_PARENT {
+public:
+	NiGeometry();
+	~NiGeometry();
+	//Run-Time Type Information
+	static const Type & TypeConst() { return TYPE; }
+private:
+	static const Type TYPE;
+public:
+	virtual void Read( istream& in, list<uint> & link_stack, unsigned int version, unsigned int user_version );
+	virtual void Write( ostream& out, map<NiObjectRef,uint> link_map, unsigned int version, unsigned int user_version ) const;
+	virtual string asString( bool verbose = false ) const;
+	virtual void FixLinks( const map<unsigned,NiObjectRef> & objects, list<uint> & link_stack, unsigned int version, unsigned int user_version );
+	virtual list<NiObjectRef> GetRefs() const;
+	virtual const Type & GetType() const;
+
+	/*!
+	 * Binds this geometry to a list of bones.  Creates and attatches a
+	 * NiSkinInstance and NiSkinData class. The bones must have a common
+	 * ancestor in the scenegraph.  This becomes the skeleton root.
+	 */
+	void BindSkin( vector< Ref<NiNode> > bone_nodes, bool bind_to_scene = false );
+	void UnbindSkin();
+	/*!
+	 * Sets the skin weights in the attached NiSkinData object.
+	 * The version on this class calculates the center and radius of
+	 * each set of affected vertices automatically.
+	 */
+	void SetBoneWeights( uint bone_index, const vector<SkinWeight> & n );
+
+	Ref<NiSkinInstance> GetSkinInstance() const;
+
+	Ref<NiGeometryData> GetData() const;
+	void SetData( const Ref<NiGeometryData> & n );
+
+	Ref<NiObject> GetUnknownLink() const;
+	void SetUnknownLink( const Ref<NiObject> & n );
+
+	string GetShader() const;
+	void SetShader( const string & n );
+
+	vector<Vector3> GetSkinInfluencedVertices() const;
+
+protected:
+	list< Ref<NiNode> > ListAncestors( const Ref<NiNode> & leaf ) const;
+	NI_GEOMETRY_MEMBERS
+private:
+	void InternalRead( istream& in, list<uint> & link_stack, unsigned int version, unsigned int user_version );
+	void InternalWrite( ostream& out, map<NiObjectRef,uint> link_map, unsigned int version, unsigned int user_version ) const;
+	string InternalAsString( bool verbose ) const;
+	void InternalFixLinks( const map<unsigned,NiObjectRef> & objects, list<uint> & link_stack, unsigned int version, unsigned int user_version );
+	list<NiObjectRef> InternalGetRefs() const;
+};
+
+}
+#endif
diff --git a/include/obj/NiGeometryData.h b/include/obj/NiGeometryData.h
new file mode 100644
index 0000000000000000000000000000000000000000..3ae06bb5dcd7c8e91b002882b6f244c22f9638dd
--- /dev/null
+++ b/include/obj/NiGeometryData.h
@@ -0,0 +1,141 @@
+/* Copyright (c) 2006, NIF File Format Library and Tools
+All rights reserved.  Please see niflib.h for licence. */
+
+#ifndef _NIGEOMETRYDATA_H_
+#define _NIGEOMETRYDATA_H_
+
+#include "NiObject.h"
+#include "../gen/SkinWeight.h"
+
+// Include structures
+#include "../Ref.h"
+namespace Niflib {
+
+// Forward define of referenced blocks
+class NiObject;
+class NiGeometryData;
+class NiSkinInstance;
+class NiNode;
+
+#include "../gen/obj_defines.h"
+
+class NiGeometryData;
+typedef Ref<NiGeometryData> NiGeometryDataRef;
+
+/*!
+ * NiGeometryData - Mesh data: vertices, vertex normals, etc.
+ */
+
+class NIFLIB_API NiGeometryData : public NI_GEOMETRY_DATA_PARENT {
+public:
+	NiGeometryData();
+	~NiGeometryData();
+	//Run-Time Type Information
+	static const Type & TypeConst() { return TYPE; }
+private:
+	static const Type TYPE;
+public:
+	virtual void Read( istream& in, list<uint> & link_stack, unsigned int version, unsigned int user_version );
+	virtual void Write( ostream& out, map<NiObjectRef,uint> link_map, unsigned int version, unsigned int user_version ) const;
+	virtual string asString( bool verbose = false ) const;
+	virtual void FixLinks( const map<unsigned,NiObjectRef> & objects, list<uint> & link_stack, unsigned int version, unsigned int user_version );
+	virtual list<NiObjectRef> GetRefs() const;
+	virtual const Type & GetType() const;
+
+	//--Counts--//
+
+	/*! Returns the number of verticies that make up this mesh.  This is also the number of normals, colors, and UV coordinates if these are used.
+	 * \return The number of vertices that make up this mesh.
+	 * \sa IShapeData::SetVertexCount
+	 */
+	int GetVertexCount() const { return int(vertices.size()); }
+
+	/*! Returns the number of texture coordinate sets used by this mesh.  For each UV set, there is a pair of texture coordinates for every vertex in the mesh.  Each set corresponds to a texture entry in the NiTexturingPropery block.
+	 * \return The number of texture cooridnate sets used by this mesh.  Can be zero.
+	 * \sa IShapeData::SetUVSetCount, ITexturingProperty
+	 */
+	short GetUVSetCount() const { return short(uvSets.size()); }
+
+	/*! Changes the number of UV sets used by this mesh.  If the new size is smaller, data at the end of the array will be lost.  Otherwise it will be retained.  The number of UV sets must correspond with the number of textures defined in the corresponding NiTexturingProperty block.
+	 * \param n The new size of the uv set array.
+	 * \sa IShapeData::GetUVSetCount, ITexturingProperty
+	 */
+	void SetUVSetCount(int n);
+
+	//--Getters--//
+
+	/*! Returns the 3D center of the mesh.
+	 * \return The center of this mesh.
+	 */
+	Vector3 GetCenter() const;
+
+	/*! Returns the radius of the mesh.  That is the distance from the center to
+	 * the farthest point from the center.
+	 * \return The radius of this mesh.
+	 */
+	float GetRadius() const;
+
+	/*! Used to retrive the vertices used by this mesh.  The size of the vector will be the same as the vertex count retrieved with the IShapeData::GetVertexCount function.
+	 * \return A vector cntaining the vertices used by this mesh.
+	 * \sa IShapeData::SetVertices, IShapeData::GetVertexCount, IShapeData::SetVertexCount.
+	 */
+	vector<Vector3> GetVertices() const { return vertices; }
+
+	/*! Used to retrive the normals used by this mesh.  The size of the vector will either be zero if no normals are used, or be the same as the vertex count retrieved with the IShapeData::GetVertexCount function.
+	 * \return A vector cntaining the normals used by this mesh, if any.
+	 * \sa IShapeData::SetNormals, IShapeData::GetVertexCount, IShapeData::SetVertexCount.
+	 */
+	vector<Vector3> GetNormals() const { return normals; }
+
+	/*! Used to retrive the vertex colors used by this mesh.  The size of the vector will either be zero if no vertex colors are used, or be the same as the vertex count retrieved with the IShapeData::GetVertexCount function.
+	 * \return A vector cntaining the vertex colors used by this mesh, if any.
+	 * \sa IShapeData::SetVertexColors, IShapeData::GetVertexCount, IShapeData::SetVertexCount.
+	 */
+	vector<Color4> GetColors() const { return vertexColors; }
+
+	/*! Used to retrive the texture coordinates from one of the texture sets used by this mesh.  The function will throw an exception if a texture set index that does not exist is specified.  The size of the vector will be the same as the vertex count retrieved with the IShapeData::GetVertexCount function.
+	 * \param index The index of the texture coordinate set to retrieve the texture coordinates from.  This index is zero based and must be a positive number smaller than that returned by the IShapeData::GetUVSetCount function.  If there are no texture coordinate sets, this function will throw an exception.
+	 * \return A vector cntaining the the texture coordinates used by the requested texture coordinate set.
+	 * \sa IShapeData::SetUVSet, IShapeData::GetUVSetCount, IShapeData::SetUVSetCount, IShapeData::GetVertexCount, IShapeData::SetVertexCount.
+	 */
+	vector<TexCoord> GetUVSet( int index ) const { return uvSets[index]; }
+	
+	//--Setters--//
+
+	/*! Used to set the vertex data used by this mesh.  Calling this function will clear all other data in this object.
+	 * \param in A vector containing the vertices to replace those in the mesh with.  Note that there is no way to set vertices one at a time, they must be sent in one batch.
+	 * \sa IShapeData::GetVertices, IShapeData::GetVertexCount
+	 */
+	virtual void SetVertices( const vector<Vector3> & in );
+
+	/*! Used to set the normal data used by this mesh.  The size of the vector must either be zero, or the same as the vertex count retrieved with the IShapeData::GetVertexCount function or the function will throw an exception.
+	 * \param in A vector containing the normals to replace those in the mesh with.  Note that there is no way to set normals one at a time, they must be sent in one batch.  Use an empty vector to signify that this mesh will not be using normals.
+	 * \sa IShapeData::GetNormals, IShapeData::GetVertexCount, IShapeData::SetVertexCount.
+	 */
+	void SetNormals( const vector<Vector3> & in );
+
+	/*! Used to set the vertex color data used by this mesh.  The size of the vector must either be zero, or the same as the vertex count retrieved with the IShapeData::GetVertexCount function or the function will throw an exception.
+	 * \param in A vector containing the vertex colors to replace those in the mesh with.  Note that there is no way to set vertex colors one at a time, they must be sent in one batch.  Use an empty vector to signify that this mesh will not be using vertex colors.
+	 * \sa IShapeData::GetColors, IShapeData::GetVertexCount, IShapeData::SetVertexCount.
+	 */
+	void SetVertexColors( const vector<Color4> & in );
+
+	/*! Used to set the texture coordinate data from one of the texture sets used by this mesh.  The function will throw an exception if a texture set index that does not exist is specified.  The size of the vector must be the same as the vertex count retrieved with the IShapeData::GetVertexCount function, or the function will throw an exception.
+	 * \param index The index of the texture coordinate set to retrieve the texture coordinates from.  This index is zero based and must be a positive number smaller than that returned by the IShapeData::GetUVSetCount function.  If there are no texture coordinate sets, this function will throw an exception.
+	 * \param in A vector containing the the new texture coordinates to replace those in the requested texture coordinate set.
+	 * \sa IShapeData::GetUVSet, IShapeData::GetUVSetCount, IShapeData::SetUVSetCount, IShapeData::GetVertexCount, IShapeData::SetVertexCount.
+	 */
+	void SetUVSet( int index, const vector<TexCoord> & in );
+
+protected:
+	NI_GEOMETRY_DATA_MEMBERS
+private:
+	void InternalRead( istream& in, list<uint> & link_stack, unsigned int version, unsigned int user_version );
+	void InternalWrite( ostream& out, map<NiObjectRef,uint> link_map, unsigned int version, unsigned int user_version ) const;
+	string InternalAsString( bool verbose ) const;
+	void InternalFixLinks( const map<unsigned,NiObjectRef> & objects, list<uint> & link_stack, unsigned int version, unsigned int user_version );
+	list<NiObjectRef> InternalGetRefs() const;
+};
+
+}
+#endif
diff --git a/include/obj/NiTriBasedGeom.h b/include/obj/NiTriBasedGeom.h
index b22d4103f0b1109f6dd597e2e37d71c9e7abe5ca..67885dfed3bb9c62f0ccf7b135b8532f4db736cf 100644
--- a/include/obj/NiTriBasedGeom.h
+++ b/include/obj/NiTriBasedGeom.h
@@ -4,18 +4,13 @@ All rights reserved.  Please see niflib.h for licence. */
 #ifndef _NITRIBASEDGEOM_H_
 #define _NITRIBASEDGEOM_H_
 
-#include "NiAVObject.h"
-#include "NiNode.h"
-#include "../gen/SkinWeight.h"
+#include "NiGeometry.h"
 
 // Include structures
 #include "../Ref.h"
 namespace Niflib {
 
 // Forward define of referenced blocks
-class NiTriBasedGeomData;
-class NiSkinInstance;
-class NiObject;
 
 #include "../gen/obj_defines.h"
 
@@ -42,34 +37,7 @@ public:
 	virtual list<NiObjectRef> GetRefs() const;
 	virtual const Type & GetType() const;
 
-	/*!
-	 * Binds this geometry to a list of bones.  Creates and attatches a
-	 * NiSkinInstance and NiSkinData class. The bones must have a common
-	 * ancestor in the scenegraph.  This becomes the skeleton root.
-	 */
-	void BindSkin( vector< Ref<NiNode> > bone_nodes, bool bind_to_scene = false );
-	void UnbindSkin();
-	/*!
-	 * Sets the skin weights in the attached NiSkinData object.
-	 * The version on this class calculates the center and radius of
-	 * each set of affected vertices automatically.
-	 */
-	void SetBoneWeights( uint bone_index, const vector<SkinWeight> & n );
-
-   void GenHardwareSkinInfo( int max_bones_per_partition = 4, int max_bones_per_vertex = 4 );
-
-	Ref<NiSkinInstance> GetSkinInstance() const;
-
-	Ref<NiTriBasedGeomData> GetData() const;
-	void SetData( const Ref<NiTriBasedGeomData> & n );
-
-	Ref<NiObject> GetUnknownLink() const;
-	void SetUnknownLink( const Ref<NiObject> & n );
-
-	string GetShader() const;
-	void SetShader( const string & n );
-
-	vector<Vector3> GetSkinInfluencedVertices() const;
+	void GenHardwareSkinInfo( int max_bones_per_partition = 4, int max_bones_per_vertex = 4 );
 
 	/*!
 	 * Generate or update a NiStringExtraData block with precalculated
@@ -78,8 +46,6 @@ public:
 	void UpdateTangentSpace();
 
 protected:
-	list< Ref<NiNode> > NiTriBasedGeom::ListAncestors( const Ref<NiNode> & leaf ) const;
-
 	NI_TRI_BASED_GEOM_MEMBERS
 private:
 	void InternalRead( istream& in, list<uint> & link_stack, unsigned int version, unsigned int user_version );
diff --git a/include/obj/NiTriBasedGeomData.h b/include/obj/NiTriBasedGeomData.h
index 7f1c083219ad4240d5e545a309feb4d2c707fc5c..df1a411d7055836b75dfd73d9ba5321a9a880ce7 100644
--- a/include/obj/NiTriBasedGeomData.h
+++ b/include/obj/NiTriBasedGeomData.h
@@ -4,7 +4,7 @@ All rights reserved.  Please see niflib.h for licence. */
 #ifndef _NITRIBASEDGEOMDATA_H_
 #define _NITRIBASEDGEOMDATA_H_
 
-#include "NiObject.h"
+#include "NiGeometryData.h"
 
 // Include structures
 #include "../Ref.h"
@@ -38,38 +38,7 @@ public:
 	virtual list<NiObjectRef> GetRefs() const;
 	virtual const Type & GetType() const;
 
-	//--Counts--//
 
-	/*! Returns the number of verticies that make up this mesh.  This is also the number of normals, colors, and UV coordinates if these are used.
-	 * \return The number of vertices that make up this mesh.
-	 * \sa IShapeData::SetVertexCount
-	 */
-	int GetVertexCount() const { return int(vertices.size()); }
-
-	/*! Returns the number of texture coordinate sets used by this mesh.  For each UV set, there is a pair of texture coordinates for every vertex in the mesh.  Each set corresponds to a texture entry in the NiTexturingPropery block.
-	 * \return The number of texture cooridnate sets used by this mesh.  Can be zero.
-	 * \sa IShapeData::SetUVSetCount, ITexturingProperty
-	 */
-	short GetUVSetCount() const { return short(uvSets.size()); }
-
-	/*! Changes the number of UV sets used by this mesh.  If the new size is smaller, data at the end of the array will be lost.  Otherwise it will be retained.  The number of UV sets must correspond with the number of textures defined in the corresponding NiTexturingProperty block.
-	 * \param n The new size of the uv set array.
-	 * \sa IShapeData::GetUVSetCount, ITexturingProperty
-	 */
-	void SetUVSetCount(int n);
-
-	//--Getters--//
-
-	/*! Returns the 3D center of the mesh.
-	 * \return The center of this mesh.
-	 */
-	Vector3 GetCenter() const;
-
-	/*! Returns the radius of the mesh.  That is the distance from the center to
-	 * the farthest point from the center.
-	 * \return The radius of this mesh.
-	 */
-	float GetRadius() const;
 
 	/*! Returns the triangle faces that make up this mesh.
 	 * \return A vector containing the triangle faces that make up this mesh.
@@ -83,58 +52,6 @@ public:
    */
    virtual void SetTriangles( const vector<Triangle> & in );
 
-	/*! Used to retrive the vertices used by this mesh.  The size of the vector will be the same as the vertex count retrieved with the IShapeData::GetVertexCount function.
-	 * \return A vector cntaining the vertices used by this mesh.
-	 * \sa IShapeData::SetVertices, IShapeData::GetVertexCount, IShapeData::SetVertexCount.
-	 */
-	vector<Vector3> GetVertices() const { return vertices; }
-
-	/*! Used to retrive the normals used by this mesh.  The size of the vector will either be zero if no normals are used, or be the same as the vertex count retrieved with the IShapeData::GetVertexCount function.
-	 * \return A vector cntaining the normals used by this mesh, if any.
-	 * \sa IShapeData::SetNormals, IShapeData::GetVertexCount, IShapeData::SetVertexCount.
-	 */
-	vector<Vector3> GetNormals() const { return normals; }
-
-	/*! Used to retrive the vertex colors used by this mesh.  The size of the vector will either be zero if no vertex colors are used, or be the same as the vertex count retrieved with the IShapeData::GetVertexCount function.
-	 * \return A vector cntaining the vertex colors used by this mesh, if any.
-	 * \sa IShapeData::SetVertexColors, IShapeData::GetVertexCount, IShapeData::SetVertexCount.
-	 */
-	vector<Color4> GetColors() const { return vertexColors; }
-
-	/*! Used to retrive the texture coordinates from one of the texture sets used by this mesh.  The function will throw an exception if a texture set index that does not exist is specified.  The size of the vector will be the same as the vertex count retrieved with the IShapeData::GetVertexCount function.
-	 * \param index The index of the texture coordinate set to retrieve the texture coordinates from.  This index is zero based and must be a positive number smaller than that returned by the IShapeData::GetUVSetCount function.  If there are no texture coordinate sets, this function will throw an exception.
-	 * \return A vector cntaining the the texture coordinates used by the requested texture coordinate set.
-	 * \sa IShapeData::SetUVSet, IShapeData::GetUVSetCount, IShapeData::SetUVSetCount, IShapeData::GetVertexCount, IShapeData::SetVertexCount.
-	 */
-	vector<TexCoord> GetUVSet( int index ) const { return uvSets[index]; }
-	
-//--Setters--//
-
-	/*! Used to set the vertex data used by this mesh.  Calling this function will clear all other data in this object.
-	 * \param in A vector containing the vertices to replace those in the mesh with.  Note that there is no way to set vertices one at a time, they must be sent in one batch.
-	 * \sa IShapeData::GetVertices, IShapeData::GetVertexCount
-	 */
-	virtual void SetVertices( const vector<Vector3> & in );
-
-	/*! Used to set the normal data used by this mesh.  The size of the vector must either be zero, or the same as the vertex count retrieved with the IShapeData::GetVertexCount function or the function will throw an exception.
-	 * \param in A vector containing the normals to replace those in the mesh with.  Note that there is no way to set normals one at a time, they must be sent in one batch.  Use an empty vector to signify that this mesh will not be using normals.
-	 * \sa IShapeData::GetNormals, IShapeData::GetVertexCount, IShapeData::SetVertexCount.
-	 */
-	void SetNormals( const vector<Vector3> & in );
-
-	/*! Used to set the vertex color data used by this mesh.  The size of the vector must either be zero, or the same as the vertex count retrieved with the IShapeData::GetVertexCount function or the function will throw an exception.
-	 * \param in A vector containing the vertex colors to replace those in the mesh with.  Note that there is no way to set vertex colors one at a time, they must be sent in one batch.  Use an empty vector to signify that this mesh will not be using vertex colors.
-	 * \sa IShapeData::GetColors, IShapeData::GetVertexCount, IShapeData::SetVertexCount.
-	 */
-	void SetVertexColors( const vector<Color4> & in );
-
-	/*! Used to set the texture coordinate data from one of the texture sets used by this mesh.  The function will throw an exception if a texture set index that does not exist is specified.  The size of the vector must be the same as the vertex count retrieved with the IShapeData::GetVertexCount function, or the function will throw an exception.
-	 * \param index The index of the texture coordinate set to retrieve the texture coordinates from.  This index is zero based and must be a positive number smaller than that returned by the IShapeData::GetUVSetCount function.  If there are no texture coordinate sets, this function will throw an exception.
-	 * \param in A vector containing the the new texture coordinates to replace those in the requested texture coordinate set.
-	 * \sa IShapeData::GetUVSet, IShapeData::GetUVSetCount, IShapeData::SetUVSetCount, IShapeData::GetVertexCount, IShapeData::SetVertexCount.
-	 */
-	void SetUVSet( int index, const vector<TexCoord> & in );
-
 protected:
 	NI_TRI_BASED_GEOM_DATA_MEMBERS
 private:
diff --git a/niflib.vcproj b/niflib.vcproj
index 36174843a60089b43aceb61e8ee5b5d171af7c00..8e2619f49d713a5a0575a730b962ee46166db7ff 100644
--- a/niflib.vcproj
+++ b/niflib.vcproj
@@ -312,7 +312,7 @@
 				MinimalRebuild="true"
 				BasicRuntimeChecks="3"
 				RuntimeLibrary="1"
-				UsePrecompiledHeader="2"
+				UsePrecompiledHeader="0"
 				PrecompiledHeaderThrough="$(ProjectDir)include\pch.h"
 				WarningLevel="3"
 				DebugInformationFormat="3"
@@ -378,7 +378,7 @@
 				AdditionalIncludeDirectories="$(ProjectDir)"
 				PreprocessorDefinitions="WIN32;NDEBUG;_CONSOLE"
 				RuntimeLibrary="0"
-				UsePrecompiledHeader="2"
+				UsePrecompiledHeader="0"
 				PrecompiledHeaderThrough="$(ProjectDir)include\pch.h"
 				WarningLevel="3"
 				DebugInformationFormat="3"
@@ -457,7 +457,7 @@
 					>
 					<Tool
 						Name="VCCLCompilerTool"
-						UsePrecompiledHeader="1"
+						UsePrecompiledHeader="0"
 						PrecompiledHeaderThrough="$(ProjectDir)include/pch.h"
 					/>
 				</FileConfiguration>
@@ -475,7 +475,7 @@
 					>
 					<Tool
 						Name="VCCLCompilerTool"
-						UsePrecompiledHeader="1"
+						UsePrecompiledHeader="0"
 						PrecompiledHeaderThrough="$(ProjectDir)include/pch.h"
 					/>
 				</FileConfiguration>
@@ -484,7 +484,7 @@
 					>
 					<Tool
 						Name="VCCLCompilerTool"
-						UsePrecompiledHeader="1"
+						UsePrecompiledHeader="0"
 						PrecompiledHeaderThrough="$(ProjectDir)include/pch.h"
 					/>
 				</FileConfiguration>
@@ -493,7 +493,7 @@
 					>
 					<Tool
 						Name="VCCLCompilerTool"
-						UsePrecompiledHeader="1"
+						UsePrecompiledHeader="0"
 						PrecompiledHeaderThrough="$(ProjectDir)include/pch.h"
 					/>
 				</FileConfiguration>
@@ -502,7 +502,7 @@
 					>
 					<Tool
 						Name="VCCLCompilerTool"
-						UsePrecompiledHeader="1"
+						UsePrecompiledHeader="0"
 						PrecompiledHeaderThrough="$(ProjectDir)include\pch.h"
 					/>
 				</FileConfiguration>
@@ -910,6 +910,14 @@
 					RelativePath=".\src\obj\NiFogProperty.cpp"
 					>
 				</File>
+				<File
+					RelativePath=".\src\obj\NiGeometry.cpp"
+					>
+				</File>
+				<File
+					RelativePath=".\src\obj\NiGeometryData.cpp"
+					>
+				</File>
 				<File
 					RelativePath=".\src\obj\NiGeomMorpherController.cpp"
 					>
@@ -1992,6 +2000,14 @@
 					RelativePath=".\include\obj\NiFogProperty.h"
 					>
 				</File>
+				<File
+					RelativePath=".\include\obj\NiGeometry.h"
+					>
+				</File>
+				<File
+					RelativePath=".\include\obj\NiGeometryData.h"
+					>
+				</File>
 				<File
 					RelativePath=".\include\obj\NiGeomMorpherController.h"
 					>
diff --git a/src/ComplexShape.cpp b/src/ComplexShape.cpp
index 67669939fb8ea34de8065b883c265ac973f1e11f..a95d09caefed65a61aaeae4b491353d98128ab62 100644
--- a/src/ComplexShape.cpp
+++ b/src/ComplexShape.cpp
@@ -226,7 +226,7 @@ void ComplexShape::Merge( const Ref<NiAVObject> & root ) {
 		propGroups[prop_group_index] = (*geom)->GetProperties();
 		
 		
-		NiTriBasedGeomDataRef geomData = (*geom)->GetData();
+		NiTriBasedGeomDataRef geomData = DynamicCast<NiTriBasedGeomData>( (*geom)->GetData() );
 
 		if ( geomData == NULL ) {
 			throw runtime_error("One of the NiTriBasedGeom found by ComplexShape::Merge with a NiTriBasedGeom has no NiTriBasedGeomData attached.");
@@ -672,7 +672,7 @@ Ref<NiAVObject> ComplexShape::Split( Ref<NiNode> & parent, Matrix44 & transform,
 		} else {
 			niData = new NiTriShapeData;
 		}
-		shapes[shape_num]->SetData( StaticCast<NiTriBasedGeomData>(niData) );
+		shapes[shape_num]->SetData( StaticCast<NiGeometryData>(niData) );
 
 		//Create a list of CompoundVertex to make it easier to
 		//test for the need to clone a vertex
diff --git a/src/gen/obj_impl.cpp b/src/gen/obj_impl.cpp
index 17bdca5e26e33bc8325cfa9f58bc80de16a33362..b78a619428c74bba7213c919c52409b4d5dade34 100644
--- a/src/gen/obj_impl.cpp
+++ b/src/gen/obj_impl.cpp
@@ -1544,7 +1544,7 @@ std::list<NiObjectRef> APSysCtlr::InternalGetRefs() const {
 	return refs;
 }
 
-void NiTriBasedGeom::InternalRead( istream& in, list<uint> & link_stack, unsigned int version, unsigned int user_version ) {
+void NiGeometry::InternalRead( istream& in, list<uint> & link_stack, unsigned int version, unsigned int user_version ) {
 	uint block_num;
 	NiAVObject::Read( in, link_stack, version, user_version );
 	NifStream( block_num, in, version );
@@ -1561,7 +1561,7 @@ void NiTriBasedGeom::InternalRead( istream& in, list<uint> & link_stack, unsigne
 	};
 }
 
-void NiTriBasedGeom::InternalWrite( ostream& out, map<NiObjectRef,uint> link_map, unsigned int version, unsigned int user_version ) const {
+void NiGeometry::InternalWrite( ostream& out, map<NiObjectRef,uint> link_map, unsigned int version, unsigned int user_version ) const {
 	NiAVObject::Write( out, link_map, version, user_version );
 	if ( data != NULL )
 		NifStream( link_map[StaticCast<NiObject>(data)], out, version );
@@ -1583,7 +1583,7 @@ void NiTriBasedGeom::InternalWrite( ostream& out, map<NiObjectRef,uint> link_map
 	};
 }
 
-std::string NiTriBasedGeom::InternalAsString( bool verbose ) const {
+std::string NiGeometry::InternalAsString( bool verbose ) const {
 	stringstream out;
 	out << NiAVObject::asString();
 	out << "  Data:  " << data << endl;
@@ -1596,9 +1596,9 @@ std::string NiTriBasedGeom::InternalAsString( bool verbose ) const {
 	return out.str();
 }
 
-void NiTriBasedGeom::InternalFixLinks( const map<unsigned,NiObjectRef> & objects, list<uint> & link_stack, unsigned int version, unsigned int user_version ) {
+void NiGeometry::InternalFixLinks( const map<unsigned,NiObjectRef> & objects, list<uint> & link_stack, unsigned int version, unsigned int user_version ) {
 	NiAVObject::FixLinks( objects, link_stack, version, user_version );
-	data = FixLink<NiTriBasedGeomData>( objects, link_stack, version );
+	data = FixLink<NiGeometryData>( objects, link_stack, version );
 	skinInstance = FixLink<NiSkinInstance>( objects, link_stack, version );
 	if ( version >= 0x0A000100 ) {
 		if ( (hasShader != 0) ) {
@@ -1607,7 +1607,7 @@ void NiTriBasedGeom::InternalFixLinks( const map<unsigned,NiObjectRef> & objects
 	};
 }
 
-std::list<NiObjectRef> NiTriBasedGeom::InternalGetRefs() const {
+std::list<NiObjectRef> NiGeometry::InternalGetRefs() const {
 	list<Ref<NiObject> > refs;
 	refs = NiAVObject::GetRefs();
 	if ( data != NULL )
@@ -1619,7 +1619,31 @@ std::list<NiObjectRef> NiTriBasedGeom::InternalGetRefs() const {
 	return refs;
 }
 
-void NiTriBasedGeomData::InternalRead( istream& in, list<uint> & link_stack, unsigned int version, unsigned int user_version ) {
+void NiTriBasedGeom::InternalRead( istream& in, list<uint> & link_stack, unsigned int version, unsigned int user_version ) {
+	NiGeometry::Read( in, link_stack, version, user_version );
+}
+
+void NiTriBasedGeom::InternalWrite( ostream& out, map<NiObjectRef,uint> link_map, unsigned int version, unsigned int user_version ) const {
+	NiGeometry::Write( out, link_map, version, user_version );
+}
+
+std::string NiTriBasedGeom::InternalAsString( bool verbose ) const {
+	stringstream out;
+	out << NiGeometry::asString();
+	return out.str();
+}
+
+void NiTriBasedGeom::InternalFixLinks( const map<unsigned,NiObjectRef> & objects, list<uint> & link_stack, unsigned int version, unsigned int user_version ) {
+	NiGeometry::FixLinks( objects, link_stack, version, user_version );
+}
+
+std::list<NiObjectRef> NiTriBasedGeom::InternalGetRefs() const {
+	list<Ref<NiObject> > refs;
+	refs = NiGeometry::GetRefs();
+	return refs;
+}
+
+void NiGeometryData::InternalRead( istream& in, list<uint> & link_stack, unsigned int version, unsigned int user_version ) {
 	uint block_num;
 	NiObject::Read( in, link_stack, version, user_version );
 	if ( version >= 0x0A020000 ) {
@@ -1699,7 +1723,7 @@ void NiTriBasedGeomData::InternalRead( istream& in, list<uint> & link_stack, uns
 	};
 }
 
-void NiTriBasedGeomData::InternalWrite( ostream& out, map<NiObjectRef,uint> link_map, unsigned int version, unsigned int user_version ) const {
+void NiGeometryData::InternalWrite( ostream& out, map<NiObjectRef,uint> link_map, unsigned int version, unsigned int user_version ) const {
 	NiObject::Write( out, link_map, version, user_version );
 	numUvSets = ushort(uvSets.size());
 	numUvSets2 = byte(uvSets.size());
@@ -1774,7 +1798,7 @@ void NiTriBasedGeomData::InternalWrite( ostream& out, map<NiObjectRef,uint> link
 	};
 }
 
-std::string NiTriBasedGeomData::InternalAsString( bool verbose ) const {
+std::string NiGeometryData::InternalAsString( bool verbose ) const {
 	stringstream out;
 	out << NiObject::asString();
 	numUvSets = ushort(uvSets.size());
@@ -1849,14 +1873,14 @@ std::string NiTriBasedGeomData::InternalAsString( bool verbose ) const {
 	return out.str();
 }
 
-void NiTriBasedGeomData::InternalFixLinks( const map<unsigned,NiObjectRef> & objects, list<uint> & link_stack, unsigned int version, unsigned int user_version ) {
+void NiGeometryData::InternalFixLinks( const map<unsigned,NiObjectRef> & objects, list<uint> & link_stack, unsigned int version, unsigned int user_version ) {
 	NiObject::FixLinks( objects, link_stack, version, user_version );
 	if ( version >= 0x14000004 ) {
 		unknownLink = FixLink<NiObject>( objects, link_stack, version );
 	};
 }
 
-std::list<NiObjectRef> NiTriBasedGeomData::InternalGetRefs() const {
+std::list<NiObjectRef> NiGeometryData::InternalGetRefs() const {
 	list<Ref<NiObject> > refs;
 	refs = NiObject::GetRefs();
 	if ( unknownLink != NULL )
@@ -1864,8 +1888,32 @@ std::list<NiObjectRef> NiTriBasedGeomData::InternalGetRefs() const {
 	return refs;
 }
 
+void NiTriBasedGeomData::InternalRead( istream& in, list<uint> & link_stack, unsigned int version, unsigned int user_version ) {
+	NiGeometryData::Read( in, link_stack, version, user_version );
+}
+
+void NiTriBasedGeomData::InternalWrite( ostream& out, map<NiObjectRef,uint> link_map, unsigned int version, unsigned int user_version ) const {
+	NiGeometryData::Write( out, link_map, version, user_version );
+}
+
+std::string NiTriBasedGeomData::InternalAsString( bool verbose ) const {
+	stringstream out;
+	out << NiGeometryData::asString();
+	return out.str();
+}
+
+void NiTriBasedGeomData::InternalFixLinks( const map<unsigned,NiObjectRef> & objects, list<uint> & link_stack, unsigned int version, unsigned int user_version ) {
+	NiGeometryData::FixLinks( objects, link_stack, version, user_version );
+}
+
+std::list<NiObjectRef> NiTriBasedGeomData::InternalGetRefs() const {
+	list<Ref<NiObject> > refs;
+	refs = NiGeometryData::GetRefs();
+	return refs;
+}
+
 void APSysData::InternalRead( istream& in, list<uint> & link_stack, unsigned int version, unsigned int user_version ) {
-	NiTriBasedGeomData::Read( in, link_stack, version, user_version );
+	NiGeometryData::Read( in, link_stack, version, user_version );
 	NifStream( hasUnknownFloats1, in, version );
 	if ( (hasUnknownFloats1 != 0) ) {
 		unknownFloats1.resize(numVertices);
@@ -1885,7 +1933,7 @@ void APSysData::InternalRead( istream& in, list<uint> & link_stack, unsigned int
 }
 
 void APSysData::InternalWrite( ostream& out, map<NiObjectRef,uint> link_map, unsigned int version, unsigned int user_version ) const {
-	NiTriBasedGeomData::Write( out, link_map, version, user_version );
+	NiGeometryData::Write( out, link_map, version, user_version );
 	NifStream( hasUnknownFloats1, out, version );
 	if ( (hasUnknownFloats1 != 0) ) {
 		for (uint i2 = 0; i2 < unknownFloats1.size(); i2++) {
@@ -1904,7 +1952,7 @@ void APSysData::InternalWrite( ostream& out, map<NiObjectRef,uint> link_map, uns
 
 std::string APSysData::InternalAsString( bool verbose ) const {
 	stringstream out;
-	out << NiTriBasedGeomData::asString();
+	out << NiGeometryData::asString();
 	out << "  Has Unknown Floats 1:  " << hasUnknownFloats1 << endl;
 	if ( (hasUnknownFloats1 != 0) ) {
 		for (uint i2 = 0; i2 < unknownFloats1.size(); i2++) {
@@ -1931,12 +1979,12 @@ std::string APSysData::InternalAsString( bool verbose ) const {
 }
 
 void APSysData::InternalFixLinks( const map<unsigned,NiObjectRef> & objects, list<uint> & link_stack, unsigned int version, unsigned int user_version ) {
-	NiTriBasedGeomData::FixLinks( objects, link_stack, version, user_version );
+	NiGeometryData::FixLinks( objects, link_stack, version, user_version );
 }
 
 std::list<NiObjectRef> APSysData::InternalGetRefs() const {
 	list<Ref<NiObject> > refs;
-	refs = NiTriBasedGeomData::GetRefs();
+	refs = NiGeometryData::GetRefs();
 	return refs;
 }
 
@@ -3664,7 +3712,7 @@ std::list<NiObjectRef> NiAmbientLight::InternalGetRefs() const {
 }
 
 void NiAutoNormalParticlesData::InternalRead( istream& in, list<uint> & link_stack, unsigned int version, unsigned int user_version ) {
-	NiTriBasedGeomData::Read( in, link_stack, version, user_version );
+	NiGeometryData::Read( in, link_stack, version, user_version );
 	if ( version <= 0x04000002 ) {
 		NifStream( numParticles, in, version );
 	};
@@ -3687,7 +3735,7 @@ void NiAutoNormalParticlesData::InternalRead( istream& in, list<uint> & link_sta
 }
 
 void NiAutoNormalParticlesData::InternalWrite( ostream& out, map<NiObjectRef,uint> link_map, unsigned int version, unsigned int user_version ) const {
-	NiTriBasedGeomData::Write( out, link_map, version, user_version );
+	NiGeometryData::Write( out, link_map, version, user_version );
 	if ( version <= 0x04000002 ) {
 		NifStream( numParticles, out, version );
 	};
@@ -3710,7 +3758,7 @@ void NiAutoNormalParticlesData::InternalWrite( ostream& out, map<NiObjectRef,uin
 
 std::string NiAutoNormalParticlesData::InternalAsString( bool verbose ) const {
 	stringstream out;
-	out << NiTriBasedGeomData::asString();
+	out << NiGeometryData::asString();
 	out << "  Num Particles:  " << numParticles << endl;
 	out << "  Size:  " << size << endl;
 	out << "  Num Active:  " << numActive << endl;
@@ -3729,12 +3777,12 @@ std::string NiAutoNormalParticlesData::InternalAsString( bool verbose ) const {
 }
 
 void NiAutoNormalParticlesData::InternalFixLinks( const map<unsigned,NiObjectRef> & objects, list<uint> & link_stack, unsigned int version, unsigned int user_version ) {
-	NiTriBasedGeomData::FixLinks( objects, link_stack, version, user_version );
+	NiGeometryData::FixLinks( objects, link_stack, version, user_version );
 }
 
 std::list<NiObjectRef> NiAutoNormalParticlesData::InternalGetRefs() const {
 	list<Ref<NiObject> > refs;
-	refs = NiTriBasedGeomData::GetRefs();
+	refs = NiGeometryData::GetRefs();
 	return refs;
 }
 
@@ -7225,26 +7273,26 @@ std::list<NiObjectRef> NiParticleRotation::InternalGetRefs() const {
 }
 
 void NiParticles::InternalRead( istream& in, list<uint> & link_stack, unsigned int version, unsigned int user_version ) {
-	NiTriBasedGeom::Read( in, link_stack, version, user_version );
+	NiGeometry::Read( in, link_stack, version, user_version );
 }
 
 void NiParticles::InternalWrite( ostream& out, map<NiObjectRef,uint> link_map, unsigned int version, unsigned int user_version ) const {
-	NiTriBasedGeom::Write( out, link_map, version, user_version );
+	NiGeometry::Write( out, link_map, version, user_version );
 }
 
 std::string NiParticles::InternalAsString( bool verbose ) const {
 	stringstream out;
-	out << NiTriBasedGeom::asString();
+	out << NiGeometry::asString();
 	return out.str();
 }
 
 void NiParticles::InternalFixLinks( const map<unsigned,NiObjectRef> & objects, list<uint> & link_stack, unsigned int version, unsigned int user_version ) {
-	NiTriBasedGeom::FixLinks( objects, link_stack, version, user_version );
+	NiGeometry::FixLinks( objects, link_stack, version, user_version );
 }
 
 std::list<NiObjectRef> NiParticles::InternalGetRefs() const {
 	list<Ref<NiObject> > refs;
-	refs = NiTriBasedGeom::GetRefs();
+	refs = NiGeometry::GetRefs();
 	return refs;
 }
 
diff --git a/src/obj/NiGeometry.cpp b/src/obj/NiGeometry.cpp
new file mode 100644
index 0000000000000000000000000000000000000000..e13997bf0acfb71f314227cf3a91dbe795d60f4a
--- /dev/null
+++ b/src/obj/NiGeometry.cpp
@@ -0,0 +1,369 @@
+/* Copyright (c) 2006, NIF File Format Library and Tools
+All rights reserved.  Please see niflib.h for licence. */
+
+#include "../../include/obj/NiGeometry.h"
+#include "../../include/obj/NiGeometryData.h"
+#include "../../include/obj/NiSkinInstance.h"
+#include "../../include/obj/NiObject.h"
+#include "../../include/obj/NiNode.h"
+#include "../../include/obj/NiSkinData.h"
+using namespace Niflib;
+
+//Definition of TYPE constant
+const Type NiGeometry::TYPE("NiGeometry", &NI_GEOMETRY_PARENT::TypeConst() );
+
+NiGeometry::NiGeometry() NI_GEOMETRY_CONSTRUCT {}
+
+NiGeometry::~NiGeometry() {}
+
+void NiGeometry::Read( istream& in, list<uint> & link_stack, unsigned int version, unsigned int user_version ) {
+	InternalRead( in, link_stack, version, user_version );
+}
+
+void NiGeometry::Write( ostream& out, map<NiObjectRef,uint> link_map, unsigned int version, unsigned int user_version ) const {
+	InternalWrite( out, link_map, version, user_version );
+}
+
+string NiGeometry::asString( bool verbose ) const {
+	return InternalAsString( verbose );
+}
+
+void NiGeometry::FixLinks( const map<unsigned,NiObjectRef> & objects, list<uint> & link_stack, unsigned int version, unsigned int user_version ) {
+	InternalFixLinks( objects, link_stack, version, user_version );
+}
+
+list<NiObjectRef> NiGeometry::GetRefs() const {
+	return InternalGetRefs();
+}
+
+const Type & NiGeometry::GetType() const {
+	return TYPE;
+};
+
+Ref<NiGeometryData> NiGeometry::GetData() const {
+	return data;
+}
+
+void NiGeometry::SetData( const Ref<NiGeometryData> & n ) {
+	data = n;
+}
+
+Ref<NiObject> NiGeometry::GetUnknownLink() const {
+	return unknownLink;
+}
+
+void NiGeometry::SetUnknownLink( const Ref<NiObject> & n ) {
+	unknownLink = n;
+}
+
+string NiGeometry::GetShader() const {
+	return shaderName;
+}
+
+void NiGeometry::SetShader( const string & n ) {
+	//Check if name is blank, if so clear shader
+	if ( n.size() == 0 ) {
+		hasShader = false;
+		shaderName.clear();
+	} else {
+		shaderName = n;
+	}
+}
+
+Ref<NiSkinInstance> NiGeometry::GetSkinInstance() const {
+	return skinInstance;
+}
+
+void NiGeometry::BindSkin( vector< Ref<NiNode> > bone_nodes, bool bind_to_scene ) {
+	//Ensure skin is not aleady bound
+	if ( skinInstance != 0 ) {
+		throw runtime_error("You have attempted to re-bind a skin that is already bound.  Unbind it first.");
+	}
+
+	//Ensure that some bones are given
+	if ( bone_nodes.size() == 0 ) {
+		throw runtime_error("You must specify at least one bone node.");
+	}
+
+	//--Find a suitable skeleton root--//
+
+	//create lists of nodes that have an influence and this TriBasedGeom
+	//as decendents
+	int num_lists = int(bone_nodes.size()) + 1;
+	vector< list<NiNodeRef> > ancestors( num_lists );
+
+	if ( GetParent() == NULL ) {
+		throw runtime_error("Attempted to bind skin on a shape with no parent.");
+	}
+
+	ancestors[bone_nodes.size()] = ListAncestors( GetParent() );
+	
+	for ( unsigned int i = 0; i < bone_nodes.size(); ++i ) {
+		NiNodeRef bonePar = bone_nodes[i]->GetParent();
+		if ( bonePar == NULL ) {
+			throw runtime_error("Attempted to bind skin to a bone with no parent.  A skeleton root cannot be a bone so all bones must have at least one parent.");
+		}
+		ancestors[i] = ListAncestors( bonePar );
+	}
+
+	if ( ancestors[0].size() == 0 ) {
+		throw runtime_error("Shape and all skin influence bones must be part of the same tree before skin bind can take place.");
+	}
+
+	NiNodeRef skeleton_root;
+   if (bind_to_scene) {
+      // Just parent to the scene
+      NiNodeRef parent = GetParent();
+      while (parent != NULL) {
+         skeleton_root = parent;
+         parent = parent->GetParent();
+      }
+
+   } else {
+      skeleton_root = ancestors[0].front();
+
+	   //Make sure bone and shapes are part of the same tree
+	   for ( int i = 1; i < num_lists; ++i ) {
+		   if ( ancestors[i].size() == 0 ) {
+			   throw runtime_error("Shape and all skin influence bones must be part of the same tree before skin bind can take place.");
+		   }
+		   if ( ancestors[i].front() != skeleton_root ) {
+			   throw runtime_error("Shape and all skin influence bones must be part of the same tree before skin bind can take place.");
+		   }
+	   }
+
+	   //Since the first items have been shown to match, pop all the stacks
+	   for ( int i = 0; i < num_lists; ++i ) {
+		   ancestors[i].pop_front();
+	   }
+
+	   //Now search for the common ancestor
+
+	   while(true) {
+		   bool all_same = true;
+		   if ( ancestors[0].size() == 0 ) {
+			   //This list is over, so the last top is the common ancestor
+			   //break out of the loop
+			   break;
+		   }
+		   NiNodeRef first_ancestor = ancestors[0].front();
+		   for ( int i = 1; i < num_lists; ++i ) {
+			   if ( ancestors[i].size() == 0 ) {
+				   //This list is over, so the last top is the common ancestor
+				   //break out of the loop
+				   all_same = false;
+				   break;
+			   }
+			   if ( ancestors[i].front() != first_ancestor ) {
+				   all_same = false;
+			   }
+		   }
+
+		   if ( all_same == true ) {
+			   //They're all the same, so set the top, pop all the stacks
+			   //and look again
+   			
+			   skeleton_root = ancestors[0].front();
+			   for ( int i = 0; i < num_lists; ++i ) {
+				   ancestors[i].pop_front();
+			   }
+		   } else {
+			   //One is different, so the last top is the common ancestor.
+			   //break out of the loop
+			   break;
+		   }
+	   }
+   }
+
+	if ( skeleton_root == NULL ) {
+		throw runtime_error("Failed to find suitable skeleton root.");
+	}
+
+	//Create a skin instance using the bone and root data
+	skinInstance = new NiSkinInstance( skeleton_root, bone_nodes );
+
+	//Create a NiSkinData object based on this mesh
+	skinInstance->SetSkinData( new NiSkinData( this ) );
+};
+
+list< Ref<NiNode> > NiGeometry::ListAncestors( const Ref<NiNode> & leaf ) const {
+	if ( leaf == NULL ) {
+		throw runtime_error("ListAncestors called with a NULL leaf NiNode Ref");
+	}
+	
+	list<NiNodeRef> ancestors;
+
+	NiNodeRef niNode = leaf;
+	while (true) {
+		ancestors.push_front(niNode);
+		if ( niNode->GetParent() == NULL ) {
+			break;
+		} else {
+			niNode = niNode->GetParent();
+		}
+	}
+
+	return ancestors;
+}
+
+void NiGeometry::UnbindSkin() {
+	//Clear skin instance
+	skinInstance = NULL;
+}
+
+vector<Vector3> NiGeometry::GetSkinInfluencedVertices() const {
+	//--Get required data & insure validity--//
+
+	NiGeometryDataRef geom_data = GetData();
+
+	if ( geom_data == NULL ) {
+		throw runtime_error("This NiGeometry has no NiGeometryData so there are no vertices to get.");
+	}
+
+	NiSkinInstanceRef skin_inst = GetSkinInstance();
+	
+
+	if ( skin_inst == NULL ) {
+		throw runtime_error("This NiGeometry is not influenced by a skin.");
+	}
+
+	NiSkinDataRef skin_data = skin_inst->GetSkinData();
+
+	if ( skin_data == NULL ) {
+		throw runtime_error("Skin Data is missing, cannot calculate skin influenced vertex position.");
+	}
+
+	//Ensure that skin instance bone count & skin data bone count match
+	if ( skin_inst->GetBoneCount() != skin_data->GetBoneCount() ) {
+		throw runtime_error("Skin Instance and Skin Data bone count do not match.");
+	}
+
+	//Get skeleton root
+	NiNodeRef skel_root = skin_inst->GetSkeletonRoot();
+	if ( skel_root == NULL ) {
+		throw runtime_error("Skin Instance is not bound to a skeleton root.");
+	}
+
+	//Get the vertices & bone nodes
+	vector<Vector3> vertices = geom_data->GetVertices();
+	vector<NiNodeRef> bone_nodes = skin_inst->GetBones();
+
+	//Set up an aray to hold the transformed vertices
+	vector<Vector3> skin_verts( vertices.size());
+
+	//Transform vertices into position based on skin data
+	Matrix44 root_world = skel_root->GetWorldTransform();
+	Matrix44 geom_world = GetWorldTransform();
+	for ( uint i = 0; i < skin_data->GetBoneCount(); ++i ) {
+		Matrix44 bone_world = bone_nodes[i]->GetWorldTransform();
+		Matrix44 bone_offset = skin_data->GetBoneTransform(i);
+		vector<SkinWeight> weights = skin_data->GetBoneWeights(i);
+		Matrix44 vert_trans =  bone_offset * bone_world;
+		for ( uint j = 0; j < weights.size(); ++j ) {
+			uint index = weights[j].index;
+			float weight = weights[j].weight;
+			skin_verts[index] += (vert_trans * vertices[index] ) * weight;
+		}
+	}
+
+	//Transform all vertices to final position
+	for ( uint i = 0; i < skin_verts.size(); ++i ) {
+		skin_verts[i] = geom_world.Inverse() * skin_verts[i];
+	}
+
+	return skin_verts;
+}
+
+// Calculate bounding sphere using minimum-volume axis-align bounding box.  Its fast but not a very good fit.
+static void CalcAxisAlignedBox(const vector<SkinWeight> & n, const vector<Vector3>& vertices, Vector3& center, float& radius)
+{
+   //--Calculate center & radius--//
+
+   //Set lows and highs to first vertex
+   Vector3 lows = vertices[ n[0].index ];
+   Vector3 highs = vertices[ n[0].index ];
+
+   //Iterate through the vertices, adjusting the stored values
+   //if a vertex with lower or higher values is found
+   for ( unsigned int i = 0; i < n.size(); ++i ) {
+      const Vector3 & v = vertices[ n[i].index ];
+
+      if ( v.x > highs.x ) highs.x = v.x;
+      else if ( v.x < lows.x ) lows.x = v.x;
+
+      if ( v.y > highs.y ) highs.y = v.y;
+      else if ( v.y < lows.y ) lows.y = v.y;
+
+      if ( v.z > highs.z ) highs.z = v.z;
+      else if ( v.z < lows.z ) lows.z = v.z;
+   }
+
+   //Now we know the extent of the shape, so the center will be the average
+   //of the lows and highs
+   center = (highs + lows) / 2.0f;
+
+   //The radius will be the largest distance from the center
+   Vector3 diff;
+   float dist2(0.0f), maxdist2(0.0f);
+   for ( unsigned int i = 0; i < n.size(); ++i ) {
+      const Vector3 & v = vertices[ n[i].index ];
+
+      diff = center - v;
+      dist2 = diff.x * diff.x + diff.y * diff.y + diff.z * diff.z;
+      if ( dist2 > maxdist2 ) maxdist2 = dist2;
+   };
+   radius = sqrt(maxdist2);
+}
+
+// Calculate bounding sphere using average position of the points.  Better fit but slower.
+static void CalcCenteredSphere(const vector<SkinWeight> & n, const vector<Vector3>& vertices, Vector3& center, float& radius)
+{
+   size_t nv = n.size();
+   Vector3 sum;
+   for (size_t i=0; i<nv; ++i)
+      sum += vertices[ n[i].index ];
+   center = sum / float(nv);
+   radius = 0.0f;
+   for (size_t i=0; i<nv; ++i){
+      Vector3 diff = vertices[ n[i].index ] - center;
+      float mag = diff.Magnitude();
+      radius = max(radius, mag);
+   }
+}
+
+void NiGeometry::SetBoneWeights( uint bone_index, const vector<SkinWeight> & n ) {
+	
+	if ( n.size() == 0 ) {
+		throw runtime_error( "You must specify at least one weight value." );
+	}
+
+	NiSkinInstanceRef skinInst = GetSkinInstance();
+
+	if ( skinInst == NULL ) {
+		throw runtime_error( "You must bind a skin before setting vertex weights.  No NiSkinInstance found." );
+	}
+
+	NiSkinDataRef skinData = skinInst->GetSkinData();
+
+	if ( skinData == NULL ) {
+		throw runtime_error( "You must bind a skin before setting vertex weights.  No NiSkinData found." );
+	}
+
+	NiGeometryDataRef geomData = GetData();
+
+	if ( geomData == NULL ) {
+		throw runtime_error( "Attempted to set weights on a mesh with no geometry data." );
+	}
+
+	//Get vertex array
+	vector<Vector3> vertices = geomData->GetVertices();
+
+   Vector3 center; float radius;
+   //CalcCenteredSphere(n, vertices, center, radius);
+   CalcAxisAlignedBox(n, vertices, center, radius);
+
+	//Translate center by bone matrix
+	center = skinData->GetBoneTransform( bone_index ) * center;
+
+	skinData->SetBoneWeights( bone_index, n, center, radius );
+}
diff --git a/src/obj/NiGeometryData.cpp b/src/obj/NiGeometryData.cpp
new file mode 100644
index 0000000000000000000000000000000000000000..82c58494863b0b876c773e5ef1b7170fd526b804
--- /dev/null
+++ b/src/obj/NiGeometryData.cpp
@@ -0,0 +1,135 @@
+/* Copyright (c) 2006, NIF File Format Library and Tools
+All rights reserved.  Please see niflib.h for licence. */
+
+#include "../../include/obj/NiGeometryData.h"
+#include "../../include/obj/NiObject.h"
+using namespace Niflib;
+
+//Definition of TYPE constant
+const Type NiGeometryData::TYPE("NiGeometryData", &NI_GEOMETRY_DATA_PARENT::TypeConst() );
+
+NiGeometryData::NiGeometryData() NI_GEOMETRY_DATA_CONSTRUCT {}
+
+NiGeometryData::~NiGeometryData() {}
+
+void NiGeometryData::Read( istream& in, list<uint> & link_stack, unsigned int version, unsigned int user_version ) {
+	InternalRead( in, link_stack, version, user_version );
+}
+
+void NiGeometryData::Write( ostream& out, map<NiObjectRef,uint> link_map, unsigned int version, unsigned int user_version ) const {
+	InternalWrite( out, link_map, version, user_version );
+}
+
+string NiGeometryData::asString( bool verbose ) const {
+	return InternalAsString( verbose );
+}
+
+void NiGeometryData::FixLinks( const map<unsigned,NiObjectRef> & objects, list<uint> & link_stack, unsigned int version, unsigned int user_version ) {
+	InternalFixLinks( objects, link_stack, version, user_version );
+}
+
+list<NiObjectRef> NiGeometryData::GetRefs() const {
+	return InternalGetRefs();
+}
+
+const Type & NiGeometryData::GetType() const {
+	return TYPE;
+};
+
+void NiGeometryData::SetUVSetCount(int n) {
+	uvSets.resize(n);
+	hasUv = ( uvSets.size() != 0 );
+}
+
+//--Setters--//
+void NiGeometryData::SetVertices( const vector<Vector3> & in ) {
+	vertices = in;
+	hasVertices = ( vertices.size() != 0 );
+
+	//Clear out all other data as it is now based on old vertex information
+	normals.clear();
+	hasNormals = false;
+	vertexColors.clear();
+	this->hasVertexColors = false;
+	for (uint i = 0; i < uvSets.size(); ++i ) {
+		uvSets[i].clear();
+	}
+
+	//If any vertices were given, calculate the new center and radius
+	//Check if there are no vertices
+	if ( vertices.size() == 0 ) {
+		center.Set(0.0f, 0.0f, 0.0f);
+		radius = 0.0f;
+		return;
+	}
+	
+	//Set lows and highs to first vertex
+	Vector3 lows = vertices[0];
+	Vector3 highs = vertices[0];
+
+	//Iterate through the rest of the vertices, adjusting the stored values
+	//if a vertex with lower or higher values is found
+	for (vector<Vector3>::const_iterator i = vertices.begin()+1; i != vertices.end(); ++i ) {
+		if ( i->x > highs.x ) highs.x = i->x;
+		else if ( i->x < lows.x ) lows.x = i->x;
+
+		if ( i->y > highs.y ) highs.y = i->y;
+		else if ( i->y < lows.y ) lows.y = i->y;
+
+		if ( i->z > highs.z ) highs.z = i->z;
+		else if ( i->z < lows.z ) lows.z = i->z;
+	}
+
+	//Now we know the extent of the shape, so the center will be the average of the lows and highs.
+	center.x = (highs.x + lows.x) / 2.0f;
+	center.y = (highs.y + lows.y) / 2.0f;
+	center.z = (highs.z + lows.z) / 2.0f;
+
+	//The radius will be the largest distance from the center
+	Vector3 diff;
+	float dist2(0.0f), maxdist2(0.0f);
+	for (vector<Vector3>::const_iterator i = vertices.begin(); i != vertices.end(); ++i ) {
+		diff = center;
+		diff.x -= i->x;
+		diff.y -= i->y;
+		diff.z -= i->z;
+		dist2 = diff.x * diff.x + diff.y * diff.y + diff.z * diff.z;
+		if ( dist2 > maxdist2 ) maxdist2 = dist2;
+	};
+	radius = sqrt(maxdist2);
+}
+
+void NiGeometryData::SetNormals( const vector<Vector3> & in ) {
+	if (in.size() != vertices.size() && in.size() != 0 )
+		throw runtime_error("Vector size must equal Vertex Count or zero.");
+	normals = in;
+	hasNormals = ( normals.size() != 0 );
+}
+
+void NiGeometryData::SetVertexColors( const vector<Color4> & in ) {
+	if (in.size() != vertices.size() && in.size() != 0 )
+		throw runtime_error("Vector size must equal Vertex Count or zero.");
+	vertexColors = in;
+	hasVertexColors = ( vertexColors.size() != 0 );
+}
+
+void NiGeometryData::SetUVSet( int index, const vector<TexCoord> & in ) {
+	if (in.size() != vertices.size())
+		throw runtime_error("Vector size must equal Vertex Count.");
+	uvSets[index] = in;
+}
+
+/*! Returns the 3D center of the mesh.
+ * \return The center of this mesh.
+ */
+Vector3 NiGeometryData::GetCenter() const {
+	return center;
+}
+
+/*! Returns the radius of the mesh.  That is the distance from the center to
+ * the farthest point from the center.
+ * \return The radius of this mesh.
+ */
+float NiGeometryData::GetRadius() const {
+	return radius;
+}
diff --git a/src/obj/NiSkinPartition.cpp b/src/obj/NiSkinPartition.cpp
index 2e0f1fcc2284d9b08bd409d8d609867af1ee825f..a45f8c8d4c5b158ec30a98f706ea5223f3f2042c 100644
--- a/src/obj/NiSkinPartition.cpp
+++ b/src/obj/NiSkinPartition.cpp
@@ -227,7 +227,7 @@ NiSkinPartition::NiSkinPartition(Ref<NiTriBasedGeom> shape) {
    if ( skinData == NULL ) {
       throw runtime_error( "You must bind a skin before setting generating skin partitions.  No NiSkinData found." );
    }
-   NiTriBasedGeomDataRef geomData = shape->GetData();
+   NiTriBasedGeomDataRef geomData = DynamicCast<NiTriBasedGeomData>( shape->GetData() );
    if ( geomData == NULL ) {
       throw runtime_error( "Attempted to generate a skin partition on a mesh with no geometry data." );
    }
@@ -376,7 +376,7 @@ NiSkinPartition::NiSkinPartition(Ref<NiTriBasedGeom> shape, int maxBonesPerParti
    if ( skinData == NULL ) {
       throw runtime_error( "You must bind a skin before setting generating skin partitions.  No NiSkinData found." );
    }
-   NiTriBasedGeomDataRef geomData = shape->GetData();
+   NiTriBasedGeomDataRef geomData = DynamicCast<NiTriBasedGeomData>(shape->GetData() );
    if ( geomData == NULL ) {
       throw runtime_error( "Attempted to generate a skin partition on a mesh with no geometry data." );
    }
diff --git a/src/obj/NiTriBasedGeom.cpp b/src/obj/NiTriBasedGeom.cpp
index 26794a9d25375632a9d3171af9b8441a5815459f..89911e9a6bae3e2b8b6e6535128f241aa19c3b55 100644
--- a/src/obj/NiTriBasedGeom.cpp
+++ b/src/obj/NiTriBasedGeom.cpp
@@ -3,12 +3,12 @@ All rights reserved.  Please see niflib.h for licence. */
 
 #include "../../include/obj/NiTriBasedGeom.h"
 #include "../../include/obj/NiTriBasedGeomData.h"
-#include "../../include/obj/NiSkinInstance.h"
-#include "../../include/obj/NiObject.h"
-#include "../../include/obj/NiSkinData.h"
 #include "../../include/obj/NiSkinPartition.h"
 #include "../../include/obj/NiExtraData.h"
 #include "../../include/obj/NiBinaryExtraData.h"
+#include "../../include/obj/NiSkinInstance.h"
+#include "../../include/obj/NiSkinData.h"
+
 using namespace Niflib;
 
 //Definition of TYPE constant
@@ -42,336 +42,6 @@ const Type & NiTriBasedGeom::GetType() const {
 	return TYPE;
 };
 
-Ref<NiTriBasedGeomData> NiTriBasedGeom::GetData() const {
-	return data;
-}
-
-void NiTriBasedGeom::SetData( const Ref<NiTriBasedGeomData> & n ) {
-	data = n;
-}
-
-Ref<NiObject> NiTriBasedGeom::GetUnknownLink() const {
-	return unknownLink;
-}
-
-void NiTriBasedGeom::SetUnknownLink( const Ref<NiObject> & n ) {
-	unknownLink = n;
-}
-
-string NiTriBasedGeom::GetShader() const {
-	return shaderName;
-}
-
-void NiTriBasedGeom::SetShader( const string & n ) {
-	//Check if name is blank, if so clear shader
-	if ( n.size() == 0 ) {
-		hasShader = false;
-		shaderName.clear();
-	} else {
-		shaderName = n;
-	}
-}
-
-Ref<NiSkinInstance> NiTriBasedGeom::GetSkinInstance() const {
-	return skinInstance;
-}
-
-void NiTriBasedGeom::BindSkin( vector< Ref<NiNode> > bone_nodes, bool bind_to_scene ) {
-	//Ensure skin is not aleady bound
-	if ( skinInstance != 0 ) {
-		throw runtime_error("You have attempted to re-bind a skin that is already bound.  Unbind it first.");
-	}
-
-	//Ensure that some bones are given
-	if ( bone_nodes.size() == 0 ) {
-		throw runtime_error("You must specify at least one bone node.");
-	}
-
-	//--Find a suitable skeleton root--//
-
-	//create lists of nodes that have an influence and this TriBasedGeom
-	//as decendents
-	int num_lists = int(bone_nodes.size()) + 1;
-	vector< list<NiNodeRef> > ancestors( num_lists );
-
-	if ( GetParent() == NULL ) {
-		throw runtime_error("Attempted to bind skin on a shape with no parent.");
-	}
-
-	ancestors[bone_nodes.size()] = ListAncestors( GetParent() );
-	
-	for ( unsigned int i = 0; i < bone_nodes.size(); ++i ) {
-		NiNodeRef bonePar = bone_nodes[i]->GetParent();
-		if ( bonePar == NULL ) {
-			throw runtime_error("Attempted to bind skin to a bone with no parent.  A skeleton root cannot be a bone so all bones must have at least one parent.");
-		}
-		ancestors[i] = ListAncestors( bonePar );
-	}
-
-	if ( ancestors[0].size() == 0 ) {
-		throw runtime_error("Shape and all skin influence bones must be part of the same tree before skin bind can take place.");
-	}
-
-	NiNodeRef skeleton_root;
-   if (bind_to_scene) {
-      // Just parent to the scene
-      NiNodeRef parent = GetParent();
-      while (parent != NULL) {
-         skeleton_root = parent;
-         parent = parent->GetParent();
-      }
-
-   } else {
-      skeleton_root = ancestors[0].front();
-
-	   //Make sure bone and shapes are part of the same tree
-	   for ( int i = 1; i < num_lists; ++i ) {
-		   if ( ancestors[i].size() == 0 ) {
-			   throw runtime_error("Shape and all skin influence bones must be part of the same tree before skin bind can take place.");
-		   }
-		   if ( ancestors[i].front() != skeleton_root ) {
-			   throw runtime_error("Shape and all skin influence bones must be part of the same tree before skin bind can take place.");
-		   }
-	   }
-
-	   //Since the first items have been shown to match, pop all the stacks
-	   for ( int i = 0; i < num_lists; ++i ) {
-		   ancestors[i].pop_front();
-	   }
-
-	   //Now search for the common ancestor
-
-	   while(true) {
-		   bool all_same = true;
-		   if ( ancestors[0].size() == 0 ) {
-			   //This list is over, so the last top is the common ancestor
-			   //break out of the loop
-			   break;
-		   }
-		   NiNodeRef first_ancestor = ancestors[0].front();
-		   for ( int i = 1; i < num_lists; ++i ) {
-			   if ( ancestors[i].size() == 0 ) {
-				   //This list is over, so the last top is the common ancestor
-				   //break out of the loop
-				   all_same = false;
-				   break;
-			   }
-			   if ( ancestors[i].front() != first_ancestor ) {
-				   all_same = false;
-			   }
-		   }
-
-		   if ( all_same == true ) {
-			   //They're all the same, so set the top, pop all the stacks
-			   //and look again
-   			
-			   skeleton_root = ancestors[0].front();
-			   for ( int i = 0; i < num_lists; ++i ) {
-				   ancestors[i].pop_front();
-			   }
-		   } else {
-			   //One is different, so the last top is the common ancestor.
-			   //break out of the loop
-			   break;
-		   }
-	   }
-   }
-
-	if ( skeleton_root == NULL ) {
-		throw runtime_error("Failed to find suitable skeleton root.");
-	}
-
-	//Create a skin instance using the bone and root data
-	skinInstance = new NiSkinInstance( skeleton_root, bone_nodes );
-
-	//Create a NiSkinData object based on this mesh
-	skinInstance->SetSkinData( new NiSkinData( this ) );
-};
-
-list< Ref<NiNode> > NiTriBasedGeom::ListAncestors( const Ref<NiNode> & leaf ) const {
-	if ( leaf == NULL ) {
-		throw runtime_error("ListAncestors called with a NULL leaf NiNode Ref");
-	}
-	
-	list<NiNodeRef> ancestors;
-
-	NiNodeRef niNode = leaf;
-	while (true) {
-		ancestors.push_front(niNode);
-		if ( niNode->GetParent() == NULL ) {
-			break;
-		} else {
-			niNode = niNode->GetParent();
-		}
-	}
-
-	return ancestors;
-}
-
-void NiTriBasedGeom::UnbindSkin() {
-	//Clear skin instance
-	skinInstance = NULL;
-}
-
-vector<Vector3> NiTriBasedGeom::GetSkinInfluencedVertices() const {
-	//--Get required data & insure validity--//
-
-	NiTriBasedGeomDataRef geom_data = GetData();
-
-	if ( geom_data == NULL ) {
-		throw runtime_error("This NiTriBasedGeom has no NiTriBasedGeomData so there are no vertices to get.");
-	}
-
-	NiSkinInstanceRef skin_inst = GetSkinInstance();
-	
-
-	if ( skin_inst == NULL ) {
-		throw runtime_error("This NiTriBasedGeom is not influenced by a skin.");
-	}
-
-	NiSkinDataRef skin_data = skin_inst->GetSkinData();
-
-	if ( skin_data == NULL ) {
-		throw runtime_error("Skin Data is missing, cannot calculate skin influenced vertex position.");
-	}
-
-	//Ensure that skin instance bone count & skin data bone count match
-	if ( skin_inst->GetBoneCount() != skin_data->GetBoneCount() ) {
-		throw runtime_error("Skin Instance and Skin Data bone count do not match.");
-	}
-
-	//Get skeleton root
-	NiNodeRef skel_root = skin_inst->GetSkeletonRoot();
-	if ( skel_root == NULL ) {
-		throw runtime_error("Skin Instance is not bound to a skeleton root.");
-	}
-
-	//Get the vertices & bone nodes
-	vector<Vector3> vertices = geom_data->GetVertices();
-	vector<NiNodeRef> bone_nodes = skin_inst->GetBones();
-
-	//Set up an aray to hold the transformed vertices
-	vector<Vector3> skin_verts( vertices.size());
-
-	//Transform vertices into position based on skin data
-	Matrix44 root_world = skel_root->GetWorldTransform();
-	Matrix44 geom_world = GetWorldTransform();
-	for ( uint i = 0; i < skin_data->GetBoneCount(); ++i ) {
-		Matrix44 bone_world = bone_nodes[i]->GetWorldTransform();
-		Matrix44 bone_offset = skin_data->GetBoneTransform(i);
-		vector<SkinWeight> weights = skin_data->GetBoneWeights(i);
-		Matrix44 vert_trans =  bone_offset * bone_world;
-		for ( uint j = 0; j < weights.size(); ++j ) {
-			uint index = weights[j].index;
-			float weight = weights[j].weight;
-			skin_verts[index] += (vert_trans * vertices[index] ) * weight;
-		}
-	}
-
-	//Transform all vertices to final position
-	for ( uint i = 0; i < skin_verts.size(); ++i ) {
-		skin_verts[i] = geom_world.Inverse() * skin_verts[i];
-	}
-
-	return skin_verts;
-}
-
-// Calculate bounding sphere using minimum-volume axis-align bounding box.  Its fast but not a very good fit.
-static void CalcAxisAlignedBox(const vector<SkinWeight> & n, const vector<Vector3>& vertices, Vector3& center, float& radius)
-{
-   //--Calculate center & radius--//
-
-   //Set lows and highs to first vertex
-   Vector3 lows = vertices[ n[0].index ];
-   Vector3 highs = vertices[ n[0].index ];
-
-   //Iterate through the vertices, adjusting the stored values
-   //if a vertex with lower or higher values is found
-   for ( unsigned int i = 0; i < n.size(); ++i ) {
-      const Vector3 & v = vertices[ n[i].index ];
-
-      if ( v.x > highs.x ) highs.x = v.x;
-      else if ( v.x < lows.x ) lows.x = v.x;
-
-      if ( v.y > highs.y ) highs.y = v.y;
-      else if ( v.y < lows.y ) lows.y = v.y;
-
-      if ( v.z > highs.z ) highs.z = v.z;
-      else if ( v.z < lows.z ) lows.z = v.z;
-   }
-
-   //Now we know the extent of the shape, so the center will be the average
-   //of the lows and highs
-   center = (highs + lows) / 2.0f;
-
-   //The radius will be the largest distance from the center
-   Vector3 diff;
-   float dist2(0.0f), maxdist2(0.0f);
-   for ( unsigned int i = 0; i < n.size(); ++i ) {
-      const Vector3 & v = vertices[ n[i].index ];
-
-      diff = center - v;
-      dist2 = diff.x * diff.x + diff.y * diff.y + diff.z * diff.z;
-      if ( dist2 > maxdist2 ) maxdist2 = dist2;
-   };
-   radius = sqrt(maxdist2);
-}
-
-// Calculate bounding sphere using average position of the points.  Better fit but slower.
-static void CalcCenteredSphere(const vector<SkinWeight> & n, const vector<Vector3>& vertices, Vector3& center, float& radius)
-{
-   size_t nv = n.size();
-   Vector3 sum;
-   for (size_t i=0; i<nv; ++i)
-      sum += vertices[ n[i].index ];
-   center = sum / float(nv);
-   radius = 0.0f;
-   for (size_t i=0; i<nv; ++i){
-      Vector3 diff = vertices[ n[i].index ] - center;
-      float mag = diff.Magnitude();
-      radius = max(radius, mag);
-   }
-}
-
-
-
-void NiTriBasedGeom::SetBoneWeights( uint bone_index, const vector<SkinWeight> & n ) {
-	
-	if ( n.size() == 0 ) {
-		throw runtime_error( "You must specify at least one weight value." );
-	}
-
-	NiSkinInstanceRef skinInst = GetSkinInstance();
-
-	if ( skinInst == NULL ) {
-		throw runtime_error( "You must bind a skin before setting vertex weights.  No NiSkinInstance found." );
-	}
-
-	NiSkinDataRef skinData = skinInst->GetSkinData();
-
-	if ( skinData == NULL ) {
-		throw runtime_error( "You must bind a skin before setting vertex weights.  No NiSkinData found." );
-	}
-
-	NiTriBasedGeomDataRef geomData = GetData();
-
-	if ( geomData == NULL ) {
-		throw runtime_error( "Attempted to set weights on a mesh with no geometry data." );
-	}
-
-	//Get vertex array
-	vector<Vector3> vertices = geomData->GetVertices();
-
-   Vector3 center; float radius;
-   //CalcCenteredSphere(n, vertices, center, radius);
-   CalcAxisAlignedBox(n, vertices, center, radius);
-
-	//Translate center by bone matrix
-	center = skinData->GetBoneTransform( bone_index ) * center;
-
-	skinData->SetBoneWeights( bone_index, n, center, radius );
-}
-
 void NiTriBasedGeom::GenHardwareSkinInfo( int max_bones_per_partition /*= 4*/, int max_bones_per_vertex /*= INT_MAX*/ ) {
    NiSkinPartitionRef skinPart; 
    if ( max_bones_per_partition == 0 ) //old method
@@ -392,27 +62,30 @@ void NiTriBasedGeom::GenHardwareSkinInfo( int max_bones_per_partition /*= 4*/, i
 }
 
 void NiTriBasedGeom::UpdateTangentSpace() {
+
+	NiTriBasedGeomDataRef niTriGeomData = DynamicCast<NiTriBasedGeomData>(this->data);
+
 	/* No data, no tangent space */
-	if( this->data == NULL ) {
-		throw runtime_error("There is no NiTriBasedGeomData attached the NiTriBasedGeom upon which UpdateTangentSpace was called.");
+	if( niTriGeomData == NULL ) {
+		throw runtime_error("There is no NiTriBasedGeomData attached the NiGeometry upon which UpdateTangentSpace was called.");
 	}
 
 	//Check if there are any UVs or Vertices before trying to retrive them
-	if ( this->data->GetUVSetCount() == 0 ) {
+	if ( niTriGeomData->GetUVSetCount() == 0 ) {
 		//There are no UVs, do nothing
 		return;
 	}
 
-	if ( this->data->GetVertexCount() == 0 ) {
+	if ( niTriGeomData->GetVertexCount() == 0 ) {
 		//There are no Vertices, do nothing
 		return;
 	}
 
 	//Get mesh data from data object
-	vector<Vector3> verts = this->data->GetVertices();
-	vector<Vector3> norms = this->data->GetNormals();
-	vector<Triangle> tris = this->data->GetTriangles();
-	vector<TexCoord> uvs = this->data->GetUVSet(0);
+	vector<Vector3> verts = niTriGeomData->GetVertices();
+	vector<Vector3> norms = niTriGeomData->GetNormals();
+	vector<Triangle> tris = niTriGeomData->GetTriangles();
+	vector<TexCoord> uvs = niTriGeomData->GetUVSet(0);
 
 	/* check for data validity */
 	if(
diff --git a/src/obj/NiTriBasedGeomData.cpp b/src/obj/NiTriBasedGeomData.cpp
index 280bdf4ee9047d1df24aaa130f7c934d8bb9a484..dec368dec615c17d5df7eeebbd4a7f8574e97e62 100644
--- a/src/obj/NiTriBasedGeomData.cpp
+++ b/src/obj/NiTriBasedGeomData.cpp
@@ -36,104 +36,6 @@ const Type & NiTriBasedGeomData::GetType() const {
 	return TYPE;
 };
 
-void NiTriBasedGeomData::SetUVSetCount(int n) {
-	uvSets.resize(n);
-	hasUv = ( uvSets.size() != 0 );
-}
-
-//--Setters--//
-void NiTriBasedGeomData::SetVertices( const vector<Vector3> & in ) {
-	vertices = in;
-	hasVertices = ( vertices.size() != 0 );
-
-	//Clear out all other data as it is now based on old vertex information
-	normals.clear();
-	hasNormals = false;
-	vertexColors.clear();
-	this->hasVertexColors = false;
-	for (uint i = 0; i < uvSets.size(); ++i ) {
-		uvSets[i].clear();
-	}
-
-	//If any vertices were given, calculate the new center and radius
-	//Check if there are no vertices
-	if ( vertices.size() == 0 ) {
-		center.Set(0.0f, 0.0f, 0.0f);
-		radius = 0.0f;
-		return;
-	}
-	
-	//Set lows and highs to first vertex
-	Vector3 lows = vertices[0];
-	Vector3 highs = vertices[0];
-
-	//Iterate through the rest of the vertices, adjusting the stored values
-	//if a vertex with lower or higher values is found
-	for (vector<Vector3>::const_iterator i = vertices.begin()+1; i != vertices.end(); ++i ) {
-		if ( i->x > highs.x ) highs.x = i->x;
-		else if ( i->x < lows.x ) lows.x = i->x;
-
-		if ( i->y > highs.y ) highs.y = i->y;
-		else if ( i->y < lows.y ) lows.y = i->y;
-
-		if ( i->z > highs.z ) highs.z = i->z;
-		else if ( i->z < lows.z ) lows.z = i->z;
-	}
-
-	//Now we know the extent of the shape, so the center will be the average of the lows and highs.
-	center.x = (highs.x + lows.x) / 2.0f;
-	center.y = (highs.y + lows.y) / 2.0f;
-	center.z = (highs.z + lows.z) / 2.0f;
-
-	//The radius will be the largest distance from the center
-	Vector3 diff;
-	float dist2(0.0f), maxdist2(0.0f);
-	for (vector<Vector3>::const_iterator i = vertices.begin(); i != vertices.end(); ++i ) {
-		diff = center;
-		diff.x -= i->x;
-		diff.y -= i->y;
-		diff.z -= i->z;
-		dist2 = diff.x * diff.x + diff.y * diff.y + diff.z * diff.z;
-		if ( dist2 > maxdist2 ) maxdist2 = dist2;
-	};
-	radius = sqrt(maxdist2);
-}
-
-void NiTriBasedGeomData::SetNormals( const vector<Vector3> & in ) {
-	if (in.size() != vertices.size() && in.size() != 0 )
-		throw runtime_error("Vector size must equal Vertex Count or zero.");
-	normals = in;
-	hasNormals = ( normals.size() != 0 );
-}
-
-void NiTriBasedGeomData::SetVertexColors( const vector<Color4> & in ) {
-	if (in.size() != vertices.size() && in.size() != 0 )
-		throw runtime_error("Vector size must equal Vertex Count or zero.");
-	vertexColors = in;
-	hasVertexColors = ( vertexColors.size() != 0 );
-}
-
-void NiTriBasedGeomData::SetUVSet( int index, const vector<TexCoord> & in ) {
-	if (in.size() != vertices.size())
-		throw runtime_error("Vector size must equal Vertex Count.");
-	uvSets[index] = in;
-}
-
-/*! Returns the 3D center of the mesh.
- * \return The center of this mesh.
- */
-Vector3 NiTriBasedGeomData::GetCenter() const {
-	return center;
-}
-
-/*! Returns the radius of the mesh.  That is the distance from the center to
- * the farthest point from the center.
- * \return The radius of this mesh.
- */
-float NiTriBasedGeomData::GetRadius() const {
-	return radius;
-}
-
 void NiTriBasedGeomData::SetTriangles( const vector<Triangle> & in ) {
    throw runtime_error("SetTriangles is not implemented for this object.");
-}
\ No newline at end of file
+}
diff --git a/include/obj/NiSkinData.h b/include/obj/NiSkinData.h
index e285af8775cdf2abd061b8e9b244798ee341bbd5..5a061aa7c7fd8fc629d0e11dd25f42b88fa14313 100644
--- a/include/obj/NiSkinData.h
+++ b/include/obj/NiSkinData.h
@@ -12,7 +12,7 @@ namespace Niflib {
 
 // Forward define of referenced blocks
 class NiSkinPartition;
-class NiTriBasedGeom;
+class NiGeometry;
 
 #include "../gen/obj_defines.h"
 
@@ -28,10 +28,10 @@ public:
 	NiSkinData();
 
 	/*!
 	 * This constructor is called by NiTriBasedGeom when it creates a new skin
 	 * instance using the BindSkin function.
 	 */
-	NiSkinData( const Ref<NiTriBasedGeom> & owner );
+	NiSkinData( const Ref<NiGeometry> & owner );
 
 	~NiSkinData();
 
@@ -28,10 +28,10 @@ public:
 
 	/*!
-	 * This constructor is called by NiTriBasedGeom when it creates a new skin
+	 * This constructor is called by NiGeometry when it creates a new skin
 	 * instance using the BindSkin function.
 	 */
diff --git a/src/obj/NiSkinData.cpp b/src/obj/NiSkinData.cpp
index 43bfa1ea607f97db6b6062f2a68a4505f2dbdd12..0187143b6bc46006a648adb728d485354831dfbd 100644
--- a/src/obj/NiSkinData.cpp
+++ b/src/obj/NiSkinData.cpp
@@ -6,7 +6,7 @@ All rights reserved.  Please see niflib.h for licence. */
 #include "../../include/gen/SkinData.h"
 #include "../../include/gen/SkinWeight.h"
 #include "../../include/obj/NiSkinPartition.h"
-#include "../../include/obj/NiTriBasedGeom.h"
+#include "../../include/obj/NiGeometry.h"
 #include "../../include/obj/NiSkinInstance.h"
 using namespace Niflib;
 
@@ -75,7 +75,7 @@ Matrix44 NiSkinData::GetOverallTransform() const {
 	return Matrix44( translation, rotation, scale );
 }
 
-NiSkinData::NiSkinData( const Ref<NiTriBasedGeom> & owner ) NI_SKIN_DATA_CONSTRUCT {
+NiSkinData::NiSkinData( const Ref<NiGeometry> & owner ) NI_SKIN_DATA_CONSTRUCT {
 	//Get skin instance
 	NiSkinInstanceRef skinInst = owner->GetSkinInstance();
 
@@ -94,7 +94,7 @@ NiSkinData::NiSkinData( const Ref<NiTriBasedGeom> & owner ) NI_SKIN_DATA_CONSTRU
 	//Get Skeleton root world transform
 	Matrix44 sr_world = skinInst->GetSkeletonRoot()->GetWorldTransform();
 
 	//Inverse owner NiTriBasedGeom matrix & multiply with skeleton root matrix
 	Matrix44 overall_mat = (owner_mat * sr_world.Inverse()).Inverse();
 
 	//Store result
@@ -109,7 +109,7 @@ NiSkinData::NiSkinData( const Ref<NiTriBasedGeom> & owner ) NI_SKIN_DATA_CONSTRU
 		//Get bone world position
 		bone_mat = bone_nodes[i]->GetWorldTransform();
 
 		//Multiply NiTriBasedGeom matrix with inversed bone matrix
 		res_mat = owner_mat * bone_mat.Inverse();
 
 		//Store result
@@ -94,7 +94,7 @@ NiSkinData::NiSkinData( const Ref<NiTriBasedGeom> & owner ) NI_SKIN_DATA_CONSTRU
 	Matrix44 sr_world = skinInst->GetSkeletonRoot()->GetWorldTransform();
 
-	//Inverse owner NiTriBasedGeom matrix & multiply with skeleton root matrix
+	//Inverse owner NiGeometry matrix & multiply with skeleton root matrix
 	Matrix44 overall_mat = (owner_mat * sr_world.Inverse()).Inverse();
 
@@ -109,7 +109,7 @@ NiSkinData::NiSkinData( const Ref<NiTriBasedGeom> & owner ) NI_SKIN_DATA_CONSTRU
 		bone_mat = bone_nodes[i]->GetWorldTransform();
 
-		//Multiply NiTriBasedGeom matrix with inversed bone matrix
+		//Multiply NiGeometry matrix with inversed bone matrix
 		res_mat = owner_mat * bone_mat.Inverse();
 